//
// C++ Implementation: VRP Generator
//
// Description: VRP-like Generator
//
//
// Author:	<Mostepha-Redounae Khouadjia>, (C) 2009
//Affiliation:	<INRIA Dolphin Team>
// Copyright: See COPYING file that comes with this distribution
//
//


#include "Generator.h"
#include "randomc.h"
#include "config.h"
#include "stocc.h"  


void manage_configuration_file(Config & config);

int main(int argc, char **argv){

Config parser("vrp.param",argv);
manage_configuration_file(parser);
Plane plane(L_PLAN,W_PLAN);
Fleet fleet(NBR_VEHICLES,MAX_CAPACITY);
Deposits depots (NBR_DEPOTS,DEPOTS_TIME_WINDOW);
CustomerSet customers (NBR_CUSTOMERS,MAX_QUANTITY,MAX_DURATION,MAX_DEPOTS_TIME_WINDOW,NBR_CLUSTERS);
Instance instanceVRP(plane,customers,depots,fleet);
instanceVRP.initialization();
instanceVRP.creation();



return EXIT_SUCCESS;

}


void manage_configuration_file(Config & config)
{

L_PLAN= config.pInt("PlaneLength");
W_PLAN= config.pInt("PlaneWidth");
NBR_CUSTOMERS = config.pInt("NbCustomers");
MAX_QUANTITY = config.pDouble("MaxQuantity");
MAX_DURATION = config.pInt("MaxDuration");
NBR_VEHICLES = config.pInt("NbVehicles");
MAX_CAPACITY = config.pInt("MaxCapacity");
CUSTOMER_DISTRIBUTION = config.pString("CustomerDistribution");
TIME_WINDOW = config.pBool("TimeWindowInstance");
STOCHASTIC= config.pBool("StochasticDemands");
NBR_DEPOTS  = config.pInt("NbDepots");
TOPOLOGY = config.pString("AreaTopology");
if(TOPOLOGY.compare("C") ==0 || TOPOLOGY.compare("Clustered") == 0 ||
   TOPOLOGY.compare("Mixed") == 0 || TOPOLOGY.compare("M") ==0)
	NBR_CLUSTERS = config.pInt("NbClusters");


DYNAMIC = config.pBool("DynamicDemands");
VRP_TYPE = config.pString("VariantVRP");
HOMOGENEOUS_DEMAND = config.pBool("HomogeneousQuantityDemands");
HOMOGENEOUS_CAPACITY = config.pBool("HomogeneousCapacities");
HOMOGENEOUS_SERVICE_DURATION = config.pBool("HomogeneousServiceDuration");
if(DYNAMIC)
	if(CUSTOMER_DISTRIBUTION.compare("Uniform") !=0 ||CUSTOMER_DISTRIBUTION.compare("U") != 0)
	
		if(CUSTOMER_DISTRIBUTION.compare("Poisson") == 0|| CUSTOMER_DISTRIBUTION.compare("P") == 0){
			ARRIVAL_FREQ = config.pDouble("RequestFrequency");
			DELTA_TIME = config.pDouble("DeltaTime");
			assert(ARRIVAL_FREQ > 0  && DELTA_TIME >0 );
			
		}
		else
			if(CUSTOMER_DISTRIBUTION.compare("Normal")==0|| CUSTOMER_DISTRIBUTION.compare("N")==0){
				NORMAL_MEAN = config.pDouble("NormalMean");
				STDEV = config.pDouble("StdDeviation");
				assert (NORMAL_MEAN  >= 1 && STDEV >=0);
			}
if(TIME_WINDOW)
	TIME_WINDOW_SIZE = config.pDouble("MaxTimeWindowSize");

if(NBR_DEPOTS >1)
 
   	for(size_t i =1; i<= NBR_DEPOTS; i ++)
      		DEPOTS_TIME_WINDOW.push_back( config.pInt("DepotTimeWindow_"+to_string(i)));
else
  		DEPOTS_TIME_WINDOW.push_back( config.pInt("DepotTimeWindow"));

MAX_DEPOTS_TIME_WINDOW = *max_element( DEPOTS_TIME_WINDOW.begin(), DEPOTS_TIME_WINDOW.end());

VISUALGNUPLOT = config.pBool("GnuplotVisualization");

DEPOTS_MOVEMENTS = config.pBool("DepotsMovements");
if(DEPOTS_MOVEMENTS) 
	 NBR_MOVES = config.pInt("NbrMovements");
VARIABLE_ROUTES = config.pBool("VariableRoutes");
if(VARIABLE_ROUTES )
	NBR_CHANGING_ROUTES = config.pInt("NbrDeviations");
VEHICLE_BREAK_DOWN = config.pBool("VehicleBreakDown");
if(VEHICLE_BREAK_DOWN)
	NBR_BREAK_DOWN= config.pInt("NbrBreaks");

/*if(VISUALGNUPLOT) 
	GNUPLOT_FILE = config.pString("GnuplotNameFile");
*/	

}

