//
// C++ Implementation: Generator
//
// Description: VRP-like Generator
//
//
// Author:  <Mostepha-Redounae Khouadjia>, (C) 2009
//
// Copyright: See COPYING file that comes with this distribution
//
//


#ifndef MULTIFILE_PROJECT
// If compiled as a single file then include these cpp files, 
// If compiled as a project then compile and link in these cpp files.
   #include "mersenne.cpp"             // code for random number generator
   #include "stoc1.cpp"                // random library source code
   #include "userintf.cpp"             // define system specific user interface
#endif

#include "Generator.h"
#include "randomc.h"
#include <cstdlib>

unsigned SEED =999;

unsigned L_PLAN,
	W_PLAN,
	NBR_CUSTOMERS,
	NBR_VEHICLES,
	MAX_DURATION,
	NBR_DEPOTS,
	MAX_DEPOTS_TIME_WINDOW,
	NBR_CLUSTERS,
	NBR_CHANGING_ROUTES,
	NBR_BREAK_DOWN,
	NBR_MOVES;

bool	TIME_WINDOW,
	STOCHASTIC,
	DYNAMIC,
	HOMOGENEOUS_DEMAND,
	HOMOGENEOUS_CAPACITY,
	HOMOGENEOUS_SERVICE_DURATION,
	DEPOTS_MOVEMENTS,
	VARIABLE_ROUTES,
	VEHICLE_BREAK_DOWN,
	VISUALGNUPLOT;

double 	
	MAX_CAPACITY,
	MAX_QUANTITY,
	ARRIVAL_FREQ,
	NORMAL_MEAN,
	STDEV,
	DELTA_TIME,
	TIME_WINDOW_SIZE;

std::vector<unsigned> DEPOTS_TIME_WINDOW;

string 	CUSTOMER_DISTRIBUTION,
	TOPOLOGY,
	VRP_TYPE;
	//GNUPLOT_FILE;

CRandomMersenne RanGen (SEED);

StochasticLib1  DGen (SEED);





bool sortFunction(  Customer  _customer1, Customer  _customer2){

	return (_customer1.getDemand().serviceTime() < _customer2.getDemand().serviceTime());

}


void sort( std::vector<Customer> & _customers){

	if(DYNAMIC && (CUSTOMER_DISTRIBUTION.compare("Uniform")==0 || CUSTOMER_DISTRIBUTION.compare("U")==0)) { 
	
		std::sort(_customers.begin(), _customers.end(), sortFunction);
	
		for(unsigned i=0 ; i < _customers.size(); i ++)
		 _customers[i].setId(i +1);

	}
}

///**************************************************************************************************************




Instance::Instance(){}

Instance::~Instance(){}

Instance::Instance( Plane & _plane,  CustomerSet &_customers,Deposits & _depots, Fleet & _vehicles ):plane(_plane), customers(_customers),depots(_depots), vehicles(_vehicles){}

void Instance::initialization(){

	depots.initialization(plane.getLength(),plane.getWidth());

	customers.initialization(plane.getLength(),plane.getWidth());

	vehicles.initialization();
}

void Instance::generateOn(ostream & _os, string& _instanceName){

	//string instanceName = "k"+ to_string(customers.size());
	_os<<"NAME: "<<_instanceName<<endl;
	_os<<"FEATURES: ";
	if(depots.size() > 1) 
		_os<<"Multi-depots ";     //MULTI-DEPOTS ";
	if(DYNAMIC)
		_os<<"Dynamic, "; //DYNAMIC ";

	if(STOCHASTIC)
		 _os<<"Stochastic, "; //STOCHASTIC ";

	_os <<"VRP with " ; //VRP WITH ";

	if(HOMOGENEOUS_DEMAND)
		_os<<"Homogeneous Demand Quantities, ";//HOMOGENEOUS DEMAND QUANTITIES ";
	else 
		_os<<"Heterogeneous Demand Quantities, ";//HETEROGENEOUS Demand Quantities"; //DEMAND QUANTITIES ";

	if(VRP_TYPE.compare("P")==0||VRP_TYPE.compare("Pick-up")==0 || VRP_TYPE.compare("p")==0)
		_os<<"Pick-up Service, " ; //WITH PICK-UP SERVICE";

	else
		if(VRP_TYPE.compare("P&D")==0||VRP_TYPE.compare("Pick-up & Delivery")==0 ||VRP_TYPE.compare("p&d")==0 )
			_os<<"and Pick-up and Delivery Service, "; //WITH PICK-UP AND DELIVERY SERVICE ";
		else 
			_os<<"Delivery Service, ";

	if(TIME_WINDOW) 
		_os<<"with Time Window, "; //WITH TIME WINDOW ";

	if(HOMOGENEOUS_CAPACITY)

		_os<<"Homogeneous Vehicle Capacity, " ; //WITH HOMOGENEOUS VEHICLE CAPACITIY ";

	else
		_os<<"Heterogeneous Vehicle Capacity, "; //WITH HETEROGENEOUS VEHICLE CAPACITY ";

	if(HOMOGENEOUS_SERVICE_DURATION)
		_os<<"and Homogeneous Service Duration. " ; //AND HOMOGENEOUS SERVICE DURATION ";
	else
		_os<<"and Heterogenous Service Duration. "; //ETEROGENEOUS SERVICE DURATION ";

	_os<<endl;

	_os<<"NBR_DEPOTS: "<<depots.size()<<endl;
	_os<<"NBR_CUSTOMERS: "<<customers.size()<<endl;;
	vehicles.printOn(_os);
	depots.printOn(_os);
	customers.printOn(_os);
	exceptionEvents(_os);
	_os<<"EOF"<<endl;

}

void Instance::creation(){

	string instanceName = "k"+ to_string(customers.size()) +".";
	if(depots.size() > 1)
		instanceName += "M"; //"md" + to_string(depots.size());//"MD-";
		
	if(DYNAMIC)
		 instanceName +=  "D"; //"d" ; //"D";
//Poisson, normal,....
	if(STOCHASTIC) 
		instanceName += "S";//"s"; //"S";

	
	/*if(TOPOLOGY.compare("Random")==0 || TOPOLOGY.compare("R")==0 || TOPOLOGY.compare("r")==0)
		instanceName += "r";
	else{
		if(TOPOLOGY.compare("Clustered")==0 || TOPOLOGY.compare("C") == 0 || TOPOLOGY.compare("c") == 0)
			instanceName += "c";
		else
			if(TOPOLOGY.compare("Mixed")==0 || TOPOLOGY.compare("M")==0 || || TOPOLOGY.compare("m")==0)
				instanceName +="m";
			
		instanceName +=	to_string(NBR_CLUSTERS)	;
	}		
	if( ! HOMOGENEOUS_DEMAND)
//		instanceName += "hq";//"HQ";
//	else 
		instanceName += "nhq";//"NHQ";

	
	if(HOMOGENEOUS_CAPACITY)
		//instanceName += "HC";
	//else
		instanceName += "nhc";//"NHC";

	if(HOMOGENEOUS_SERVICE_DURATION)
		//instanceName += "hd";//"HSD";
	//else
		instanceName += "nhsd";//NHSD";
*/
	if(VRP_TYPE.compare("P")==0||VRP_TYPE.compare("Pick-up")==0)
		instanceName += "P"; //pu";//"P";
	else
		if(VRP_TYPE.compare("P&D")==0||VRP_TYPE.compare("Pick-up & Delivery")==0)
		instanceName += "P&D";//"p&d";//"P&D";
	
	if(TIME_WINDOW)
		 instanceName += "TW"; //"tw";//"TW";

	instanceName += ".VRP"; //".vrp";//".VRP";

	std::ofstream fileOut (instanceName.c_str());
	writingTest(fileOut);

 	//sort(customers.begin(), customers.end(), sortFunction);
	generateOn(fileOut,instanceName);
	fileOut.close();
	generateOn(cout,instanceName);	

	if(VISUALGNUPLOT){
		std::ofstream fileGnuplot ("vrp.visual");
		writingTest(fileGnuplot);
		visualGnuplot(fileGnuplot);
		std::system("gnuplot script.gnu");  
		fileGnuplot.close();
				
	}
	
}


void Instance::exceptionEvents(ostream& _os){

	if(DEPOTS_MOVEMENTS){
		_os<<	"DEPOTS_MOVEMENTS_SECTION"	<<endl;
		_os<<"ID"<<'\t'<<"XCOOR"<<'\t'	<<"YCOORD"<<'\t'<<"MOVING_TIME"<<	endl;

		for(unsigned i =0; i < NBR_MOVES; i ++){

			double xCoord = RanGen.IRandom(-(L_PLAN/2),L_PLAN/2);

			double yCoord = RanGen.IRandom(-(W_PLAN/2),W_PLAN/2);

			_os<<	RanGen.IRandom(1,NBR_DEPOTS)<<'\t'<<
				xCoord<<'\t'<<
				yCoord<<'\t'<<
				RanGen.IRandom(1,MAX_DEPOTS_TIME_WINDOW)<<endl;
		}
	}

	if(VEHICLE_BREAK_DOWN){
		_os<<"VEHICLES_BREAK_DOWN_SECTION"	<<endl;
		_os<<"ID"<<'\t'<<"BREAK_DOWN_TIME"<<	endl;
		for(unsigned i = 0; i < NBR_BREAK_DOWN; i++)
			
			_os<<RanGen.IRandom(1,NBR_VEHICLES)<<'\t'<< RanGen.IRandom(1,MAX_DEPOTS_TIME_WINDOW)<<endl;
	}

	if(VARIABLE_ROUTES){
		_os<<"VARIABLE_ROUTES_SECTION"<<endl;
		_os<<"CUSTOMER"	<<'\t'<<"CUSTOMER"<<'\t'<<	"UNFEASIBLE_TIME"<<	endl;
		for(unsigned i = 0; i< NBR_CHANGING_ROUTES; i++){
			unsigned C1 = RanGen.IRandom(1,NBR_CUSTOMERS);
			unsigned C2 = RanGen.IRandom(1,NBR_CUSTOMERS);
			while (C1==C2) C2 = RanGen.IRandom(1,NBR_CUSTOMERS);
			_os<<C1<<'\t'<<'\t'<<C2<<'\t'<<'\t'<<RanGen.IRandom(1,MAX_DEPOTS_TIME_WINDOW)<<endl;

		}
	}
}
	
	
		

void Instance::visualGnuplot(ostream & _os ){

	
	unsigned i =0;
	_os<<"#index "<<i++<<endl;
	for(size_t j = 0; j < depots.size(); j++)
		depots[j].printOn(_os);

	
	_os<<endl<<endl<<"#index "<<i++<<endl;
	for(size_t j=0; j < customers.size(); j++)
		customers[j].printOn(_os);		
	
	



}

/*plan::PrintOn(ostream & _os)
{
_os*/


///**************************************************************************************************************


Plane::Plane(){}

Plane::~Plane(){}

Plane::Plane(const Plane & _plane) {

	length = _plane.length; 

	width = _plane.width;
}

Plane::Plane(double _length , double _width):length(_length), width(_width){}

double Plane::getLength(){return length;}

double Plane::getWidth(){return width;}


//void Plane::setDualAxis(double _length , double _width): {length = _length; width= _width}

void Plane::printOn(ostream & _os){
	
 _os<<'\t'<<length<<'\t'<<width<<endl;

}





///******************************************************************************************************************
unsigned Vehicle::nextNumVehicle= 1; 

Vehicle::Vehicle(){id = nextNumVehicle++;}

Vehicle::~Vehicle(){}

Vehicle::Vehicle(double _capacity, double _travelDuration, double _velocity):
capacity(_capacity),travelDuration(_travelDuration), velocity(_velocity){id = nextNumVehicle++;}

Vehicle::Vehicle(const Vehicle & _vehicle){
	id = _vehicle.id;
	capacity = _vehicle.capacity;
	travelDuration = _vehicle.travelDuration;
	velocity = _vehicle.velocity;
}

void Vehicle::setCapacity(double _capacity){ capacity = _capacity;}

void Vehicle::printOn(ostream & _os){

	_os<<id<<'\t'<<capacity<<'\t'<<endl;

	//travelDuration<<'\t'<<velocity<<endl;
}



///******************************************************************************************************************


Fleet::Fleet(){}

Fleet::Fleet(unsigned _nbrVehicles, double _maxCapacity):nbrVehicles(_nbrVehicles),maxCapacity(_maxCapacity){}

Fleet::~Fleet(){}

double Fleet::getMaxCapacity(){return maxCapacity;}

void Fleet::addVehicle(const Vehicle & _vehicle) {push_back(_vehicle);}

void Fleet::addVehicle(double _capacity, double _travelDuration, double _velocity){

	Vehicle vehicle (_capacity, _travelDuration, _velocity);
	push_back(vehicle);

}

void Fleet::initialization(){

	for(size_t i =0; i< nbrVehicles ; i++){
  		
		Vehicle vehicle(maxCapacity);

		if(! HOMOGENEOUS_CAPACITY)

			vehicle.setCapacity(RanGen.IRandom(1,maxCapacity));

  		addVehicle(vehicle);
	}
}

void Fleet::printOn(ostream & _os){
	_os	<<"NBR_VEHICLES: "	<< nbrVehicles	<<endl;;
	_os	<<"VEHICLE_MAX_CAPACITY: "	<<maxCapacity	<<endl;

	if(! HOMOGENEOUS_CAPACITY){

	_os<<"VEHICLES_SECTION"<<endl;
	
	_os	<<"ID"<<'\t'
		<<"CAPACITY"<<'\t'
		<<endl;

	for(size_t i =0 ; i < this -> size() ; i++)
		this->operator[](i).printOn(_os);


	}
}

///******************************************************************************************************************
unsigned Depot::nextNumDepot= 1;

Depot::Depot(){}

Depot::~Depot(){}

Depot::Depot(double _xcoord, double _ycoord, double _leftTW, double _rightTW):xcoord(_xcoord), ycoord(_ycoord),leftTW(_leftTW), rightTW(_rightTW){id = nextNumDepot++ ;}

void Depot::setCoordinate(double _ycoord, double _xcoord){ ycoord = _ycoord; xcoord = _xcoord; }

void Depot::printOn(ostream & _os){

	_os	<<id<<'\t'
		<<xcoord<<'\t'
		<<ycoord<<'\t'<<"[ "
		<<leftTW<<" , "
		<<rightTW<<" ]"<<endl;
}


///******************************************************************************************************************

Deposits::Deposits(){}

Deposits::Deposits(unsigned _nbrDeposits, std::vector<unsigned>_timeWindowDepots):nbrDeposits(_nbrDeposits),timeWindowDepots(_timeWindowDepots){}

Deposits::~Deposits(){}

void Deposits::addDepot(const Depot& _depot) {

	 push_back(_depot);
}

void Deposits::addDepot(const double _xcoord, const double _ycoord, const double _leftTW, const double _rightTW  ){

	Depot depot(_xcoord,_ycoord,_leftTW,_rightTW);
  	push_back(depot);

}

void Deposits::initialization(double _lengthPlane, double _widthPlane){

	for(size_t i =0; i< nbrDeposits; i++){

  		double xCoord = RanGen.IRandom(-(_lengthPlane/2),_lengthPlane/2);
  		double yCoord = RanGen.IRandom(-(_widthPlane/2),_widthPlane/2);
  		addDepot(xCoord, yCoord, 0.0, timeWindowDepots[i]);

	}

}

void Deposits::printOn(ostream & _os){

	_os	<<"DEPOTS SECTION "<<endl;
	_os	<<"ID"<<'\t'
		<<"XCOORD"<<'\t'
		<<"YCOORD"<<'\t'
		<<"TIME_WINDOW"<<endl;

	for(size_t i = 0 ; i < this->size() ; i++)
   
      		this->operator[](i).printOn(_os);

}

///******************************************************************************************************************

Demand::Demand():minQuantity(0),maxQuantity(0),timeService(0),serviceDuration(0), leftTW(0), rightTW(0){}

Demand::~Demand(){}

Demand::Demand(const Demand & _demand){

	minQuantity=  _demand.minQuantity;
	maxQuantity = _demand.maxQuantity;
	timeService = _demand.timeService;
	serviceDuration = _demand.serviceDuration;
	leftTW = _demand.leftTW;
	rightTW = _demand.rightTW;
}

Demand::Demand	(double _quantity, double _timeService, double _serviceDuration, double _leftTW, double _rightTW):
		minQuantity(_quantity),
		maxQuantity(_quantity),
		timeService(_timeService),
		serviceDuration(_serviceDuration),
		leftTW(_leftTW), 
		rightTW(_rightTW){}


Demand & Demand::operator=(const Demand & _demand){
	minQuantity = _demand.minQuantity;
	maxQuantity = _demand.maxQuantity;
	timeService = _demand.timeService;
	serviceDuration = _demand.serviceDuration;
	leftTW = _demand.leftTW;
	rightTW = _demand.rightTW;
	return *this;
}


void Demand::setQuantity(double _quantity ){maxQuantity = minQuantity = _quantity;}

void Demand::setRangeQuantity(double _min, double _max){
	minQuantity=_min;
	maxQuantity=_max;
}

void Demand::setTimeService(double _timeService){timeService=_timeService;}

double Demand::serviceTime(){return timeService;}

void Demand::setServiceDuration(double _serviceDuration){serviceDuration=_serviceDuration;}

void Demand::setTimeWindow(double _leftTw, double _rightTw){
	leftTW=_leftTw;
	rightTW=_rightTw;
} 


void Demand::printOn(ostream& _os){

	if(minQuantity==maxQuantity)
		_os	<<maxQuantity<<'\t'
			<<timeService<<'\t'<<'\t'
			<<serviceDuration<<'\t'<<'\t'<<"[ "
			<<leftTW<<" , "
			<<rightTW<<" ]"<<endl;
	else 			

		_os	<<"[ "<<minQuantity<<" , "
			<<maxQuantity<<" ]"<<'\t'
			<<timeService<<'\t'<<'\t'
			<<serviceDuration<<'\t'<<'\t'<<"[ "
			<<leftTW<<" , "
			<<rightTW<<" ]"<<endl;

}



///******************************************************************************************************************

unsigned Customer::nextNumCustomer= 1; 

Customer::Customer(){id=nextNumCustomer++;}

Customer::~Customer(){}

Customer::Customer(double _xcoord, double _ycoord, const Demand& _demand): xcoord(_xcoord), ycoord(_ycoord),demand(_demand)

{id=nextNumCustomer++;}

Customer::Customer(double _xcoord, double _ycoord):xcoord(_xcoord), ycoord(_ycoord),demand(){id=nextNumCustomer++;}


Customer::Customer(double _lengthPlane, double _widthPlane, double _xCoorCentroid, double _yCoordCentroid, double _clusterDensity){

	double xLimit = _lengthPlane / 2;
	double  yLimit = _widthPlane / 2 ;
	xcoord = RanGen.IRandom (std::max(_xCoorCentroid -(_clusterDensity * _lengthPlane) ,  - xLimit),
				std::min(_xCoorCentroid + ( _clusterDensity * _lengthPlane) , xLimit) ) ;

	ycoord = RanGen.IRandom (std::max(_yCoordCentroid -(_clusterDensity * _widthPlane) ,  - yLimit),
				std::min(_yCoordCentroid + ( _clusterDensity * _widthPlane) , yLimit) ) ;

	id=nextNumCustomer++;	
}

void Customer::setId(unsigned _id){id = _id;}

double Customer::xCoordinate(){return xcoord;}

double Customer::yCoordinate(){return ycoord;}

void Customer::setCoordinates(double _lengthPlane, double _widthPlane){

	xcoord = RanGen.IRandom(-(_lengthPlane/2),_lengthPlane/2);
	ycoord = RanGen.IRandom(-(_widthPlane/2),_widthPlane/2);
}

void Customer::setCoordinates(double _lengthPlane, double _widthPlane, double _xCoorCentroid, double _yCoordCentroid, double _clusterDensity){

	double xLimit = _lengthPlane / 2;
	double  yLimit = _widthPlane / 2 ;
	xcoord = RanGen.IRandom (std::max(_xCoorCentroid -(_clusterDensity * _lengthPlane) ,  - xLimit),
				std::min(_xCoorCentroid + ( _clusterDensity * _lengthPlane) , xLimit) ) ;
	ycoord = RanGen.IRandom (std::max(_yCoordCentroid -(_clusterDensity * _widthPlane) ,  - yLimit),
				std::min(_yCoordCentroid + ( _clusterDensity * _widthPlane) , yLimit) ) ;
}

void Customer::printOn(ostream & _os){

	_os	<<id<<'\t'
		<<xcoord<<'\t'
		<<ycoord<<'\t';

	demand.printOn(_os);

}

void Customer::setCoordinate(double _xcoord, double _ycoord){
	xcoord = _xcoord; 
	ycoord = _ycoord;
 }

void Customer::setDemand(const Demand&  _demand){ demand = _demand; }

Demand&  Customer::getDemand(){return demand;}
///******************************************************************************************************************


CustomerSet::CustomerSet(){}

CustomerSet::CustomerSet(unsigned _nbrCustomers, double _maxQuantityDemand,double _maxServiceDuration, unsigned 				_lengthWorkingDay, unsigned _nbClusters):
 			nbrCustomers(_nbrCustomers),
			maxQuantityDemand(_maxQuantityDemand),
			maxServiceDuration(_maxServiceDuration),
			lengthWorkingDay(_lengthWorkingDay),
			nbClusters(_nbClusters){}

CustomerSet::~CustomerSet(){}

void CustomerSet::addCustomer(const Customer& _customer){

push_back(_customer);

}

void CustomerSet::addCustomer(double _xcoord, double _ycoord, Demand _demand){

	Customer customer(_xcoord,_ycoord,_demand);
	push_back(customer);

}

void CustomerSet::addCustomer(Customer& _customer, double _quantity, double _timeService, double _serviceDuration,double _leftTW, double _rightTW ){

 	Demand demand(_quantity,_timeService,_serviceDuration,_leftTW,_rightTW);
	_customer.setDemand(demand);
 	push_back(_customer);

}

void CustomerSet::initialization(double _lengthPlane, double _widthPlane){

	double  xCoord, yCoord, xLimit, yLimit;
	xLimit = _lengthPlane / 2;
	yLimit = _widthPlane / 2;
	double clusterDensity = 0.1 ;
	unsigned MaxNeighbors, clusterSize ;
	if((TOPOLOGY.compare("Random")==0) || TOPOLOGY.compare("R")==0)
		clusterSize = nbrCustomers;
	else{

		if ((TOPOLOGY.compare("Clustered")==0 || TOPOLOGY.compare("C")==0)) 
			clusterSize = nbrCustomers/ nbClusters; 
		else

		if ((TOPOLOGY.compare("Mixed") == 0 )|| TOPOLOGY.compare("M")==0)
	
			clusterSize = nbrCustomers/ (nbClusters+1) ; 

		else{
			std::cerr<<"Unknown Topology....Indicate the topology of service area!!!!!"<<endl;

			exit(0);		
	        }

		for(size_t i = 0; i <nbClusters; i++){

			Customer centroidCustomer(RanGen.IRandom(-xLimit, xLimit) , RanGen.IRandom(-yLimit, yLimit)); 
   			addCustomer(centroidCustomer);
			MaxNeighbors = RanGen.IRandom(1,clusterSize);
		
			if( ((TOPOLOGY.compare("Clustered")==0 || TOPOLOGY.compare("C")==0)) && (i == nbClusters -1) ) 
				MaxNeighbors =  nbrCustomers - (this -> size());
		
			for(size_t j = 0; j < MaxNeighbors -1 ; j++){
	
				Customer customer(_lengthPlane, 
						_widthPlane, 
						centroidCustomer.xCoordinate(), 
						centroidCustomer.yCoordinate(),
	 					clusterDensity);
 
				addCustomer(customer); 

			}
		}
	}

 	 while(this -> size() < nbrCustomers){    ///For Random distribution of Customers in Service area

		xCoord = RanGen.IRandom(-xLimit , xLimit);

		yCoord = RanGen.IRandom(-yLimit, yLimit);
		
		Customer customer(xCoord, yCoord);

		addCustomer(customer);

    	}


	for(size_t i = 0; i < nbrCustomers; i++)
	{
	
		Demand demand;

		if(HOMOGENEOUS_SERVICE_DURATION)
	
			demand.setServiceDuration(maxServiceDuration);
		
		else
			demand.setServiceDuration(RanGen.IRandom(0,maxServiceDuration));
	
		if(DYNAMIC)
			if(CUSTOMER_DISTRIBUTION.compare("Uniform")==0 || CUSTOMER_DISTRIBUTION.compare("U")==0)
				
				 demand.setTimeService(RanGen.IRandom(1,MAX_DEPOTS_TIME_WINDOW));
			
				
				
				
			
		
			else{
	
				double probaDistrib;

				if(CUSTOMER_DISTRIBUTION.compare("Poisson")==0 ||CUSTOMER_DISTRIBUTION.compare("P")==0)

			 		probaDistrib = DGen.Poisson(DELTA_TIME / ARRIVAL_FREQ); ///Lambda is the number of 							requests per time unit 
				else{  
	
					if(CUSTOMER_DISTRIBUTION.compare("Normal")==0||CUSTOMER_DISTRIBUTION.compare("N")==0)

						probaDistrib  =  DGen.Normal(NORMAL_MEAN, STDEV);
					else{
						std::cerr<<"Unkown Distribution for Dynamic Orders"<<endl;
			
						exit(0);			
					}
				}

				
				if(i >  1)  
					demand.setTimeService(probaDistrib + this->operator[](i-1).getDemand().serviceTime());
				else
		
					demand.setTimeService(max(1.00,probaDistrib));  ///Just for the first customer
						
			}			
		else
	
			demand.setTimeService(0);

	
		double minQt = RanGen.IRandom(1,MAX_QUANTITY);

		double maxQt = RanGen.IRandom(minQt,MAX_QUANTITY);
		
		
		if(VRP_TYPE.compare("Pick-up")== 0||VRP_TYPE.compare("P")== 0)
			
			
			if(STOCHASTIC)  ///The stochastic demands are not Homogeneous by definition

				demand.setRangeQuantity( -maxQt, -minQt);

			else  

				if(HOMOGENEOUS_DEMAND)
				
					demand.setQuantity(- MAX_QUANTITY);

				else

					demand.setQuantity(- RanGen.IRandom(1,MAX_QUANTITY));

		

		else
			if(VRP_TYPE.compare("Pick-up & Delivery")== 0 ||VRP_TYPE.compare("P&D")== 0)

	
				if(STOCHASTIC) ///The stochastic demands are not Homogeneous by definition

					if(RanGen.IRandom(0,1)) 
										
						demand.setRangeQuantity(minQt, maxQt);
										
					else

						demand.setRangeQuantity(- maxQt, - minQt);
					
					
				else

					if(HOMOGENEOUS_DEMAND)

						if(RanGen.IRandom(0,1))

							demand.setQuantity(MAX_QUANTITY);
						else
							demand.setQuantity(- MAX_QUANTITY);
	
					else{
									
						if(RanGen.IRandom(0,1)) 
										
							demand.setQuantity(RanGen.IRandom(1,MAX_QUANTITY));
										
						else

							demand.setQuantity(- RanGen.IRandom(1,MAX_QUANTITY));
					
					}
	
					
			else	/// Delivery  or Capacited case
				if(STOCHASTIC) ///The stochastic demands are not Homogeneous in quantity by definition

					demand.setRangeQuantity(minQt,maxQt);
				else
					
					if(HOMOGENEOUS_DEMAND)
					
						demand.setQuantity(MAX_QUANTITY);	

					else
					
						demand.setQuantity(RanGen.IRandom(1,MAX_QUANTITY));


	

	if(TIME_WINDOW)	{
		
		unsigned  windowSize = RanGen.IRandom(0,TIME_WINDOW_SIZE); ///we have the possibility to define a size of the windows
		
		if(!DYNAMIC){
			unsigned  midTime = RanGen.IRandom(0,MAX_DEPOTS_TIME_WINDOW);
		
			demand.setTimeWindow(midTime, min((midTime + windowSize),MAX_DEPOTS_TIME_WINDOW));
		}
		else
			 
			demand.setTimeWindow(demand.serviceTime(), min((int)(demand.serviceTime() + windowSize) ,(int)MAX_DEPOTS_TIME_WINDOW) );		
	}


	this->operator[](i).setDemand(demand);
	
 	  
	}

        sort(*this);			//sort(this->begin(), this ->end(), sortFunction); 

	
	
}






void CustomerSet::printOn(ostream & _os){

	_os<<"CUSTOMERS SECTION"<<endl;

	_os 	<<"ID"<<'\t'
		<<"XCOORD"<<'\t'
		<<"YCOORD"<<'\t'
		<<"QT"<<'\t'
		<<"TIME_SERV"<<'\t'
		<<"DURATION"<<'\t'
		<<"TIME_WINDOW"<<endl;

	for(size_t i = 0; i < this ->size(); i++)
	
		this->operator[](i).printOn(_os);
		


}



///******************************************************************************************************************

		






