//
// C++ Interface: Generator
//
// Description: 
//
//
// Author:  <>, (C) 2009
//
// Copyright: See COPYING file that comes with this distribution
//
//

#ifndef Generator
#define Generator

#include <iostream> 
#include <fstream>
#include<vector>
#include <algorithm>
#include "Tools.h"
#include "randomc.h"
#include "stocc.h"
#include "config.h"
#include <assert.h>



using namespace std;


extern unsigned SEED,
		L_PLAN,
		W_PLAN,
		NBR_CUSTOMERS,
		NBR_VEHICLES,
		MAX_DURATION,
		NBR_DEPOTS,
		MAX_DEPOTS_TIME_WINDOW,
		NBR_CLUSTERS,
		NBR_BREAK_DOWN,
		NBR_CHANGING_ROUTES,
		NBR_MOVES;

extern bool	TIME_WINDOW,
		STOCHASTIC,
		DYNAMIC,
		HOMOGENEOUS_DEMAND,
		HOMOGENEOUS_CAPACITY,
		HOMOGENEOUS_SERVICE_DURATION,
		DEPOTS_MOVEMENTS,
		VEHICLE_BREAK_DOWN,
		VARIABLE_ROUTES,
		VISUALGNUPLOT;
		


extern double	MAX_CAPACITY,
		MAX_QUANTITY,
		ARRIVAL_FREQ,
		NORMAL_MEAN,
		STDEV,
		DELTA_TIME,
		TIME_WINDOW_SIZE;

extern std::vector<unsigned> DEPOTS_TIME_WINDOW;		

extern string 	CUSTOMER_DISTRIBUTION,
		TOPOLOGY,
		VRP_TYPE;
		//GNUPLOT_FILE;



class Plane{
	public:

		Plane();
		~Plane();
		Plane(double _lenght , double width);
		Plane (const Plane & _plane); 
		//void setDualAxis(double _length , double _widthdouble Plane::length(){return length;}
		double getLength();
		double getWidth();
		void printOn(ostream & _os);
	private:
		double length,
			width;
}; 


class Vehicle {

	public:

		Vehicle(); 
		Vehicle(double _capacity=0.0, double _travelDuration=0.0, double _velocity=1.0);		
		Vehicle(const Vehicle & _vehicle);
		void setCapacity(double _capacity);
		void printOn(ostream & _os);
		~Vehicle();

	private:
		static unsigned nextNumVehicle;
		unsigned id, capacity;
		double travelDuration, velocity;

};

class Fleet:public vector<Vehicle> {

	public:

		Fleet();
		Fleet(unsigned _nbrVehicles,double _maxCapacity=0.0);
		~Fleet();
		double getMaxCapacity();
		void addVehicle(const Vehicle & _vehicle);
		void addVehicle(double _capacity=0.0, double _travelDuration=0.0, double _velocity=1.0);
		void initialization();
		void printOn(ostream & _os);
	private:
		double	maxCapacity,
			maxDuration;
		unsigned nbrVehicles;

};


class Depot{

	public:

		Depot();
		~Depot();
		Depot(double _xcoord, double _ycoord, double _leftTW=0.0, double _rightTW=0.0);
		void setCoordinate(double _ycoord, double _xcoord);
		void printOn(ostream & _os);

	private:
		unsigned id;
		double	xcoord,
			ycoord,
			leftTW,
			rightTW;
		static unsigned nextNumDepot;
};


class Deposits : public std::vector<Depot> {
	public:
 		Deposits();
 		Deposits(unsigned _nbrDeposits, std::vector<unsigned>_timeWindowDepots);
		~Deposits();
		void addDepot(const Depot& _depot);
		void addDepot(const double _xcoord, const double _ycoord, const double _leftTW = 0.0, const double _rightTW =0.0);
		void initialization(double _lengthPlane, double _widthPlane);
		void printOn(ostream & _os);

	private:

		unsigned nbrDeposits;
		std::vector<unsigned> timeWindowDepots;

};

class Demand{

	public:

		Demand();
		Demand(const Demand & _demand);
		Demand(double _quantity , double _timerService=0.0, double _serviceDuration=0.0, double _leftTW=0.0, double _rightTW=0.0);
		Demand & operator=(const Demand & _demand);
		void setQuantity(double _quantity);
		void setRangeQuantity(double _min, double _max);
		double serviceTime();
		void setTimeService(double _timeService);
		void setServiceDuration(double _serviceDuration);
		void setTimeWindow(double _leftTw, double _rightTw); 
		void initialize();
		void printOn(ostream & _os);
		~Demand();
	private:
		double	minQuantity,
			maxQuantity,
			timeService,
			serviceDuration,
			leftTW,
			rightTW;
};


class Customer{

	public:

		Customer();
		Customer(double _xcoord, double _ycoord, const Demand& _customerDemand);
		Customer(double _xcoord, double _ycoord);
		Customer(double _lengthPlane, double _widthPlane, double _xCoorCentroid, double _yCoordCentroid, double _clusterDensity);
		void setId(unsigned _id);
		void setCoordinates(double _lengthPlane, double _widthPlane);
		void setCoordinates(double _lengthPlane, double _widthPlane, double _xCoorCentroid, double _yCoordCentroid, double _clusterDensity);
		void printOn(ostream & _os);
		void setCoordinate(double _xcoord, double _ycoord);
		void setDemand(const Demand& _demand);
		Demand & getDemand();
		double xCoordinate();
		double yCoordinate();
		~Customer();

	private:
		unsigned id;
		Demand demand;
		double	xcoord,
			ycoord;
		static unsigned nextNumCustomer;
 
};


class CustomerSet : public std::vector<Customer>
{

	public:
		CustomerSet();
		CustomerSet(unsigned _nbrCustomers,double _maxQuantityDemandDemand =1.0, double _maxServiceDuration=0.0, unsigned _lengthWorkingDay = 0, unsigned _nbClusters = 1);
		~CustomerSet();
		void addCustomer(const Customer& _customer); 
		void addCustomer(double _xcoord, double _ycoord, Demand _demand);
		void addCustomer(Customer& _customer, double _quantity, double _timeService, double _serviceDuration, double _leftTW, double _rightTW );
		void initialization(double _lengthPlane, double _widthPlane);
		void printOn(ostream & _os);
	private:
		double	maxQuantityDemand,
			maxServiceDuration;
		unsigned nbrCustomers,
			lengthWorkingDay,
			nbClusters;

};




class Instance{

	public:

		Instance();
		~Instance();
		Instance(Plane & _plane, CustomerSet &_customers,  Deposits & _depots, Fleet & _vehicles );
		void generateOn(ostream & _os, std::string& _instanceName);
		void creation();
		void initialization();
		void exceptionEvents(ostream & _os);
		void visualGnuplot(ostream & _os);

	private:

		Plane plane;
		Deposits depots;
		CustomerSet customers;
		Fleet vehicles;

};

bool sortFunction(  Customer  _customer1, Customer  _customer2); /// for customer vector

void sort( std::vector<Customer>& _customers) ; 

#endif

